c******************************************************************************
c***               File mit Gitterkonstanten anlegen                        ***
c******************************************************************************

c     Harald Ritter
c     Institut fuer Kristallographie
c     der Universitaet Tuebingen
c     Charlottenstrasse 33
c     D-72070 Tuebingen
c     Germany
c     Tel: +49(0)7071/29-76389
c     E-mail: harald.ritter@uni-tuebingen.de

c     Last updated: 15 Jan 1999


      program mettab

      parameter(max = 16, NATS = 64)
      logical filex
      real a(9), shift, stdd(9), werte(max)
      real atom(5,NATS), datom(5,NATS)
      character line*132, form*20, text(9)*23, char(max)*132,
     *          datform*24, par(5)*20, txt*23, atext(NATS)*4
      character*80 outfile, cycfile, datfile, inpfile, title

      data text /'  A       Delt     Stdd','  B       Delt     Stdd',
     *           '  C       Delt     Stdd','Alpha     Delt     Stdd',
     *           ' Beta     Delt     Stdd','Gamma     Delt     Stdd',
     *           ' q1       Delt     Stdd',' q2       Delt     Stdd',
     *           ' q3       Delt     Stdd'/
      data par / ',x     Delt     Stdd',',y     Delt     Stdd',
     *           ',z     Delt     Stdd',',B     Delt     Stdd',
     *           ',N     Delt     Stdd'/
      data pi / 3.14159265359 /
      data ka / 1 /


c --- Name des Inputfiles eingeben:

c      write(*,'(2x,$a)') 'Please type the input filename: '
      read(*,'(a)') inpfile

c === Input-File oeffnen:

      open(ka,file=inpfile,status='old',err=1000)

c --- Titel lesen:

      ist = 0
      call getinp(ka,'TIT','eee',werte,char,max,num,nn,ist)
      if(ist.eq.11) goto 1100
      title(1:80) = char(1)(1:80)

c --- Name des Datenfiles lesen:

      ist = 0
      call getinp(ka,'DAT','eee',werte,char,max,num,nn,ist)
      if(ist.eq.11) goto 1100
      datfile(1:80) = char(1)(1:80)

c --- Phaseninformation einlesen:

      ist = 1
      call getinp(ka,'PHS','eee',werte,char,max,num,nn,ist)
      if(ist.eq.11) goto 1100
      imo = nint(werte(2))

c --- Gitterkonstanten einlesen:

      ist = 1
      call getinp(ka,'MET','eee',werte,char,max,num,nn,ist)
      if(ist.eq.11) goto 1100
      do 10 i=1,6
         a(i) = werte(i)
         stdd(i) = 0.
 10   continue

c --- Wellenvektor einlesen:

      if(imo.eq.1) then
        ist = 1
        call getinp(ka,'WAV','eee',werte,char,max,num,nn,ist)
        if(ist.eq.11) goto 1100
        do 15 i=1,3
           a(6+i) = werte(i)
           stdd(i) = 0.
 15     continue
      end if

c --- Atomparameter einlesen:

      natom = 0
      ist = 1
 22   call getinp(ka,'ATO','eee',werte,char,max,num,nn,ist)
      if(ist.ne.11.and.ist.ne.13) then
         natom = natom + 1
         atext(natom)(1:4) = char(1)(1:4)
         do 24 i=1,5
            atom(i,natom)  = werte(3+i)
            datom(i,natom) = 0.
 24      continue
         goto 22
      end if

      close(ka)

c === Zyklus-File bestimmen, oeffnen und lesen:

      call outfil(cycfile,inpfile,'cyc',laenge,0)
      open(ka,file=cycfile,status='old',err=1200)

      nz = 6
      if(imo.eq.1) nz = 9

c --- Gitterkonstanten suchen und einlesen:

      do 50 i=1,nz

         rewind(ka)
 20      read(ka,'(a)',end=50,err=1300) line
         ix = index(line,text(i))
         if(ix.eq.0) goto 20

 30      read(ka,'(a)',err=1300) line
         if(index(line,'|').gt.0) goto 30

         backspace(ka)
         backspace(ka)
         form = '(   x,(f10.0,2f9.0))'
         write(form(2:4),'(i3)') ix - 4 
         read(ka,fmt=form,err=1300) a(i), shift, stdd(i)

 50   continue

c --- Atomparameter suchen und einlesen:

      do 70 k=1,natom

         txt(1:3) = atext(k)(1:3)

         do 60 i=1,5 

            txt(4:23) = par(i)(1:20)

            rewind(ka)
 55         read(ka,'(a)',end=60,err=1300) line
            ix = index(line,txt)
            if(ix.eq.0) goto 55

 57         read(ka,'(a)',err=1300) line
            if(index(line,'|').gt.0) goto 57
            
            backspace(ka)
            backspace(ka)
            form = '(   x,(f10.0,2f9.0))'
            write(form(2:4),'(i3)') ix - 3
            read(ka,fmt=form,err=1300) atom(i,k), shift, datom(i,k)
 60      continue
 70   continue

c --- Zyklusfile schliessen:

      close(ka)

c --- Volumen berechnen:

      cosa = cos(pi/180. * a(4))
      cosb = cos(pi/180. * a(5))
      cosg = cos(pi/180. * a(6))
      sina = sqrt(1. - cosa * cosa)
      sinb = sqrt(1. - cosb * cosb)
      sing = sqrt(1. - cosg * cosg)
      vs = 1. + 2. * cosa * cosb * cosg - cosa**2 - cosb**2 - cosg**2
      dvol = (stdd(1)/a(1))**2 + (stdd(2)/a(2))**2 + (stdd(3)/a(3))**2 +
     *       (stdd(4)/vs * sina * (cosa - cosb*cosg))**2 +
     *       (stdd(5)/vs * sinb * (cosb - cosa*cosg))**2 +
     *       (stdd(6)/vs * sing * (cosg - cosa*cosb))**2
      vol  = a(1) * a(2) * a(3) * sqrt(vs)
      dvol = vol * sqrt(dvol)

c --- Temperatur aus dem Datenfile lesen:

      open(ka,file=datfile,status='old',err=1600)
 80   ist = 1
      call getinp(ka,'   ','   ',werte,char,max,num,nn,ist)
      if(ist.ne.1) goto 1700
      if(char(1)(1:3).ne.'DUO'.and.char(1)(1:3).ne.'duo'.and.
     *   char(1)(1:3).ne.'DAT'.and.char(1)(1:3).ne.'dat'.and.
     *   char(1)(1:3).ne.'NEU'.and.char(1)(1:3).ne.'neu') goto 80
      if(num.ne.8) goto 80
      temp = werte(6)
      close(ka)

c --- Gitterkonstanten in das Ausgabe-File schreiben:

      datform = '(f8.2,  (1x,f10.5,f9.5))'
      write(datform(7:8),'(i1)') nz + 1
      call outfil(outfile,inpfile,'met',laenge,0)

      inquire(file=outfile,exist=filex)
      if(filex) then
        open(ka,file=outfile,status='old',err=1800)
 120    read(ka,*,end=200,err=1900)
        goto 120
      else
        open(ka,file=outfile,status='unknown',err=1800)
        write(ka,'(a78)') title
        write(ka,'(a)') 'TITELENDE     FOR'
        if(imo.eq.1) then
          write(ka,'(4a)') 
     *'    T/K       a        da         b        db',
     *'         c        dc       Alpha    dAlpha     Beta',
     *'     dBeta      Gamma    dGamma     q1       dq1',
     *'         q2       dq2       q3       dq3        Vol       dV'   
        else
          write(ka,'(3a)') 
     *    '    T/K       a        da         b        db',
     *    '         c        dc       Alpha    dAlpha     Beta',
     *    '     dBeta      Gamma    dGamma     Vol       dV'
        end if
      end if
 200  write(ka,datform) temp,(a(i),stdd(i),i=1,nz),vol,dvol
      close(ka)

c --- Atomparameter abspeichern:

      do 300 k=1,natom
         outfile = ' '
         n = 0
         do 210 i=1,4
            if(atext(k)(i:i).ne.' ') then
               n = n + 1
               outfile(n:n) = atext(k)(i:i)
            end if
 210     continue
         outfile(n+1:n+4) = '.ato'
         inquire(file=outfile,exist=filex)
         if(filex) then
            open(ka,file=outfile,status='old',err=1800)
 220        read(ka,*,end=250,err=1900)
            goto 220
         else
            open(ka,file=outfile,status='unknown',err=1800)
            write(ka,'(a,a4)') 'Parameters of atom ',atext(k)
            write(ka,'(a78)') title
            write(ka,'(a)') 'TITELENDE     FOR'
            write(ka,'(a,a)') 
     *           '    T/K     x        dx       y        dy       z',
     *           '        dz       B        dB       N        dN'
         endif
 250     write(ka,'(f8.2,10(1x,f8.5))') 
     *        temp,(atom(i,k),datom(i,k),i=1,5)
         close(ka)
 300  continue

      goto 2000


c --- Fehlermeldungen:

 1000 write(*,'(2x,a)') 'Cannot open input file'
      goto 2000
 1100 write(*,'(2x,a)') 'Error reading input file'
      goto 2000
 1200 write(*,'(2x,a)') 'Cannot open cycle file'
      goto 2000
 1300 write(*,'(2x,a)') 'Error reading cycle file'
      goto 2000
 1600 write(*,'(2x,a)') 'Cannot open data file'
      goto 2000
 1700 write(*,'(2x,a)') 'Error reading data file'
      goto 2000
 1800 write(*,'(2x,a)') 'Cannot open output file'
      goto 2000
 1900 write(*,'(2x,a)') 'Error reading output file'


c --- Ende:

 2000 continue
      end

c******************************************************************************
c***          Eine Zeile einer Datei mit oder ohne Code einlesen            ***
c******************************************************************************

      subroutine getinp(ke,code,ecod,werte,char,max,num,nn,ist)

c     ke:      Kanalnummer
c     code:    Zeilencode, z. B. 'TIT'
c              code = '   ': Zeile ohne Code einlesen
c     ecod:    Code, bei dem das Einlesen abgebrochen wird
c     char:    Daten als Character-String abgespeichert
c     werte:   Daten als Realwerte abgespeichert
c     ist:     Eingabe: = 0 : Zeichenkette einlesen --> char
c                       = 1 : Daten einer Zeile als Realwerte (werte) und
c                             zusaetzlich als character (char) abspeichern 
c              Ausgabe: = 11 : Codewort nicht gefunden!
c                       = 13 : Ende beim Einlesen gefunden!
c                       = 15 : Zu viele Werte in der Eingabezeile (num > max)!
c     nn:      Zeilennummer
c     line:    Eingelesene Eingabe-Zeile
c              (einschliesslich aller Fortsetzungszeilen)
c     LMAX     Maximale Anzahl der Zeichen pro Zeile 
c     MAXLINE: Maximale Anzahl der Zeichen pro Eingabe-Zeile 
c              (einschliesslich der Fortsetzungszeilen)
c     max:     Maximale Anzahl der Werte pro Eingabe-Zeile
c     num:     Anzahl der gefundenen Werte pro Eingabe-Zeile


      parameter(MAXLINE = 500, LMAX = 250)
      real werte(max)
      character zeile*(LMAX), code*3, ecod*3, char(max)*132
      character line*(MAXLINE)


c -- Initialisierung:

      ibeg = 1

      do 5 k=1,max
         char(k)  = ' '
         werte(k) = 0.
5     continue

 10   k = 0
      line = ' '
      num = 0

c --- Zeile einlesen:

 11   zeile = ' '
      read(ke,'(a)',end=1100) zeile
c      write(*,'(a,a)') 'Z = ',zeile(1:70)
      nn = nn + 1

c --- Nach Fortsetzungszeilen suchen.

      do 20 i=1,LMAX
         if(zeile(i:i).eq.'\\') then
           if(i.eq.1) then
             goto 11
           else
             if(zeile(i-1:i-1).eq.' ') goto 11
           end if
         end if
         k = k + 1
         if(k.gt.MAXLINE) goto 30
         line(k:k) = zeile(i:i)
 20   continue

c --- Nach der Kennung suchen:

 30   if(code.ne.'   ') then

        if(ecod.ne.'eee'.and.line(1:3).eq.ecod) then
          backspace(ke)
          goto 1300
        end if
        if(line(1:3).ne.code(1:3)) goto 10

c --- ">" suchen:

        do 100 i=1,MAXLINE
           if(line(i:i).eq.'>') then
             ibeg = i + 1
             goto 110
           end if
100     continue
        stop 'ERROR: ">" is missing'

      end if

c --- Anfang suchen:

110   do 120 i=ibeg,MAXLINE
         if(line(i:i).ne.' ') then
           ibeg = i
           goto 130
         end if
120   continue    
      return 

c --- Daten der Zeile abspeichern:

130   if(ist.eq.0) then
        char(1)(1:132) = line(ibeg:ibeg+132)
      else
        ic    = 0
        n     = 0
        num   = 1
        ibl   = 0
        komma = 0
        do 250 i=ibeg,MAXLINE
           if(line(i:i).eq.' ') then
             ibl = ibl + 1
           else if(line(i:i).eq.',') then
             komma = 1
             num = num + 1
             n = 0
           else
             if(ibl.gt.0.and.komma.eq.0) then
               num = num + 1
               n = 0
             end if
             ic = ic + 1
             n = n + 1
             char(num)(n:n) = line(i:i)
             ibl = 0
             komma = 0
           end if
250     continue
        if(num.gt.max) goto 1500
        do 300 l=1,num
           read(char(l)(1:15),'(bn,f15.0)',err=300) werte(l)
300     continue
      end if

      return

c --- Fehlermeldungen und Warnungen zurueckgeben:
 
1100  ist = 11
      return
1300  ist = 13
      return
1500  ist = 15
      return

      end


C******************************************************************************
C***            Name der Ausgabe-Datei erzeugen                             ***
C******************************************************************************

      subroutine outfil(name,file,filspe,laenge,nr)

      parameter(MAX = 80)
      character filspe*3, file*80, name*80, form*4


c --- Initialisierung:

      name = ' '

c --- Zehnerpotenz der Zahl NR bestimmen:

      nz = 0
      if(nr.gt.0) nz = int(alog10(real(nr))) + 1

c --- Ende suchen:

      do 50 i = MAX,1,-1
         if(file(i:i).eq.'.') then
           iend = i - 1
           goto 60
         end if
50    continue

c --- Anfang suchen:

60    do 100 i = MAX,1,-1
         if(file(i:i).eq.'/'.or.file(i:i).eq.']'.or.
     *        file(i:i).eq.'\\') then
           ibeg = i + 1
           goto 120
         end if
100   continue
      ibeg = 1

c --- Basisnamen abspeichern:
      
120   n = 0
      do 200 i = ibeg,iend
         if(file(i:i).ne.' ') then
           n = n + 1
           name(n:n) = file(i:i)
         end if
200   continue     
    
c --- Laufende Nummer und Extension anhaengen:

      form = '(i )'
      write(form(3:3),'(i1)') nz
      if(nz.gt.0) write(name(n+1:n+nz),form) nr
      name(n+nz+1:n+nz+1) = '.'
      name(n+nz+2:n+nz+4) = filspe(1:3)
      laenge = n + nz + 4

      return
      end
